#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "lcd.h"



/* 
** constants/macros 
*/

/*
** constant definitions
*/
static const PROGMEM unsigned char degree[] =
{

	0b00000,
	0b00000,
	0b01001,
	0b01001,
	0b01001,
	0b01111,
	0b01000,
	0b10000
	 // 
};

#define DDR(x) (*(&x - 1))      /* address of data direction register of port x */
#if defined(__AVR_ATmega64__) || defined(__AVR_ATmega128__)
    /* on ATmega64/128 PINF is on port 0x00 and not 0x60 */
    #define PIN(x) ( &PORTF==&(x) ? _SFR_IO8(0x00) : (*(&x - 2)) )
#else
	#define PIN(x) (*(&x - 2))    /* address of input register of port x          */
#endif


#define lcd_e_delay()   __asm__ __volatile__( "rjmp 1f\n 1:" );
#define lcd_e_high()    LCD_E_PORT  |=  _BV(LCD_E_PIN);
#define lcd_e_low()     LCD_E_PORT  &= ~_BV(LCD_E_PIN);
#define lcd_e_toggle()  toggle_e()
#define lcd_rs_high()   LCD_RS_PORT |=  _BV(LCD_RS_PIN)
#define lcd_rs_low()    LCD_RS_PORT &= ~_BV(LCD_RS_PIN)

/*
** local functions
*/



/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
static inline void _delayFourCycles(uint16_t __count)
{
    if ( __count == 0 )    
        __asm__ __volatile__( "rjmp 1f\n 1:" );    // 2 cycles
    else
        __asm__ __volatile__ (
    	    "1: sbiw %0,1" "\n\t"                  
    	    "brne 1b"                              // 4 cycles/loop
    	    : "=w" (__count)
    	    : "0" (__count)
    	   );
}


/************************************************************************* 
delay for a minimum of <us> microseconds
the number of loops is calculated at compile-time from MCU clock frequency
*************************************************************************/
#define delay(us)  _delayFourCycles( F_CPU/4000*(us)/1000)




/*************************************************************************
Low-level function to write byte to LCD controller
Input:    data   byte to write to LCD
          rs     1: write data    
                 0: write instruction
Returns:  none
*************************************************************************/
static void lcd_write(uint8_t data,uint8_t rs) 
{   
	if (rs) {   /* write data        (RS=1, RW=0) */
       lcd_rs_high();
    } else {    /* write instruction (RS=0, RW=0) */
       lcd_rs_low();
    }
	
	LCD_PORT = (LCD_PORT & 0xF0) | ((data & 0xF0)/16);

	delay(100);
	lcd_e_high();
    delay(10);
	lcd_e_low();
	delay(100);

	LCD_PORT = (LCD_PORT & 0xF0) | (data & 0xF);

	delay(100);
	lcd_e_high();
    delay(10);
	lcd_e_low();
	delay(100);

}




/*
** PUBLIC FUNCTIONS 
*/

/*************************************************************************
Send LCD controller instruction command
Input:   instruction to send to LCD controller, see HD44780 data sheet
Returns: none
*************************************************************************/
void lcd_command(uint8_t cmd)
{
    lcd_write(cmd,0);
}

/*************************************************************************
Send data byte to LCD controller 
Input:   data to send to LCD controller, see HD44780 data sheet
Returns: none
*************************************************************************/
void lcd_output(uint8_t data)
{
    lcd_write(data,1);
}


/*************************************************************************
Print a character to LCD controller 
Input:   data to send to LCD controller, see HD44780 data sheet
Returns: none
*************************************************************************/
void lcd_putc(char data)
{
	if (data == '') { 
		lcd_write(0,1);
	} else { 
    	lcd_write(data,1);
    }
}



/*************************************************************************
Set cursor to specified position
Input:    x  horizontal position  (0: left most position)
          y  vertical position    (0: first line)
Returns:  none
*************************************************************************/
void lcd_gotoxy(uint8_t x, uint8_t y)
{

    if ( y==0 ) 
        lcd_command(0x80+x);
    else
        lcd_command(0x80+0x40+x);


}/* lcd_gotoxy */

/*************************************************************************
Clear display and set cursor to home position
*************************************************************************/
void lcd_clrscr(void)
{
    lcd_command(1<<LCD_CLR);
	delay(1640);
}


/*************************************************************************
Set cursor to home position
*************************************************************************/
void lcd_home(void)
{
    lcd_command(1<<LCD_HOME);
}


/*************************************************************************
Display string without auto linefeed 
Input:    string to be displayed
Returns:  none
*************************************************************************/
void lcd_puts(const char *s)
/* print string on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = *s++) ) {
        lcd_putc(c);
    }

}/* lcd_puts */

/*************************************************************************
Set the cursor control and de display on. 
Input:    Blink and show flag of cursor
Returns:  none
*************************************************************************/

void lcd_set_cursor_control(uint8_t blink, uint8_t show)
{
	uint8_t command = 0b00001100;
	if (blink) command |= 2;
	if (show) command |= 1;
	lcd_command(command);
}
/*************************************************************************
Display string from program memory without auto linefeed 
Input:     string from program memory be be displayed                                        
Returns:   none
*************************************************************************/
void lcd_puts_p(const char *progmem_s)
/* print string from program memory on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = pgm_read_byte(progmem_s++)) ) {
        lcd_putc(c);
    }

}/* lcd_puts_p */


void lcd_sendNibble(uint8_t nibble){
	
	LCD_PORT = (LCD_PORT & 0xF0) | nibble;

	delay(1000);
	lcd_e_high();
    delay(10);
	lcd_e_low();
	delay(100);
	LCD_PORT &= 0xF0 ;
}


/*************************************************************************
Initialize display and select type of cursor 
Returns:  none
*************************************************************************/
void lcd_init()
{
    /*
     *  Initialize LCD to 4 bit I/O mode
     */
	int i;

 	LCD_DATA_DDR |= 0x0F;

	LCD_RS_DDR |= (1 << LCD_RS_PIN);
	LCD_E_DDR |= (1 << LCD_E_PIN);

    delay(16000);        /* wait 16ms or more after power-on       */
    
    /* initial write to lcd is 8bit */
	lcd_rs_low();

	lcd_sendNibble(0x3);
    delay(32000);         /* delay, busy flag can't be checked here */
   	delay(32000); 
    
	/* repeat last command */ 
    lcd_sendNibble(0x3);
    delay(1000);           /* delay, busy flag can't be checked here */
    
    /* repeat last command a third time */
    lcd_sendNibble(0x3);
    delay(640);           /* delay, busy flag can't be checked here */


	/** Switch to 4 bit mode */
	lcd_sendNibble(0x2);
    delay(640);           /* delay, busy flag can't be checked here */


 //   delay(64);           /* some displays need this additional delay */
    /* from now the LCD only accepts 8 bit I/O, we can use lcd_command() */    

 //   lcd_command(LCD_FUNCTION_DEFAULT);      /* function set: display lines  */
	lcd_command(0b00101100); // 4Bit 2Rows 
	delay(40);
	lcd_command(0b00001111); // Display on, Cursor on, blink on
	delay(40); 
    lcd_clrscr();                           /* display clear                */  
	lcd_command(_BV(6));  /* set CG RAM start address 0 */
    for(i=0; i<8; i++){
        lcd_output(pgm_read_byte_near(&degree[i]));
    }


}/* lcd_init */
