#ifndef LCD_H
#define LCD_H


#if (__GNUC__ * 100 + __GNUC_MINOR__) < 303
#error "This library requires AVR-GCC 3.3 or later, update to newer AVR-GCC compiler !"
#endif

#include <inttypes.h>
#include <avr/pgmspace.h>


#define LCD_DATA_DDR     DDRC
#define LCD_PORT         PORTC        /**< port for the LCD lines   */
#define LCD_RS_PORT      PORTC        /**< port for RS line         */
#define LCD_RS_DDR       DDRC
#define LCD_RS_PIN       4            /**< pin  for RS line         */
#define LCD_E_PORT       PORTC        /**< port for Enable line     */
#define LCD_E_DDR        DDRC
#define LCD_E_PIN        5            /**< pin  for Enable line     */

/**
 *  @name Definitions for LCD command instructions
 *  The constants define the various LCD controller instructions which can be passed to the
 *  function lcd_command(), see HD44780 data sheet for a complete description.
 */

/* instruction register bit positions, see HD44780U data sheet */
#define LCD_CLR               0      /* DB0: clear display                  */
#define LCD_HOME              1      /* DB1: return to home position        */

/* display on/off, cursor on/off, blinking char at cursor position */
#define LCD_DISP_OFF             0x08   /* display off                            */
#define LCD_DISP_ON              0x0C   /* display on, cursor off                 */
#define LCD_DISP_ON_BLINK        0x0D   /* display on, cursor off, blink char     */
#define LCD_DISP_ON_CURSOR       0x0E   /* display on, cursor on                  */
#define LCD_DISP_ON_CURSOR_BLINK 0x0F   /* display on, cursor on, blink char      */





/**
  Initialize display and select type of cursor
*/
extern void lcd_init();


/**
  Clear display and set cursor to home position
*/
extern void lcd_clrscr(void);


/**
  Set cursor to home position
*/
extern void lcd_home(void);


/**
  Set cursor to specified position

 @param    x horizontal position\n (0: left most position)
 @param    y vertical position\n   (0: first line)
*/
extern void lcd_gotoxy(uint8_t x, uint8_t y);

/**
  Display character at current cursor position
 @param    write a byte without any transformation
*/
extern void lcd_output(uint8_t out); 

/**
  Display character at current cursor position
 @param    c character to be displayed
*/
extern void lcd_putc(char c);


/**
  Display string without auto linefeed
 @param    s string to be displayed
*/
extern void lcd_puts(const char *s);


/**
  Display string from program memory without auto linefeed
 @param    s string from program memory be be displayed
 @return   none
 @see      lcd_puts_P
*/
extern void lcd_puts_p(const char *progmem_s);


/**
  Send LCD controller instruction command
 @param    cmd instruction to send to LCD controller, see HD44780 data sheet
*/
extern void lcd_command(uint8_t cmd);


/**
  Set cursor control

 @param    blink
 @param    show

*/

extern void lcd_set_cursor_control(uint8_t blink, uint8_t show);

/**
 macros for automatically storing string constant in program memory
*/
#define lcd_puts_P(__s)         lcd_puts_p(PSTR(__s))

/*@}*/
#endif //LCD_H
